@@ -14,7 +14,6 @@
 #include "postgres.h"
 
 #include "jit/llvmjit.h"
-#include "jit/llvmjit_backport.h"
 #include "jit/llvmjit_emit.h"
 
 #include "miscadmin.h"
@@ -30,9 +29,6 @@
 #include <llvm-c/BitWriter.h>
 #include <llvm-c/Core.h>
 #include <llvm-c/ExecutionEngine.h>
-#if LLVM_VERSION_MAJOR > 16
-#include <llvm-c/Transforms/PassBuilder.h>
-#endif
 #if LLVM_VERSION_MAJOR > 11
 #include <llvm-c/Orc.h>
 #include <llvm-c/OrcEE.h>
@@ -42,18 +38,14 @@
 #endif
 #include <llvm-c/Support.h>
 #include <llvm-c/Target.h>
-#if LLVM_VERSION_MAJOR < 17
 #include <llvm-c/Transforms/IPO.h>
 #include <llvm-c/Transforms/PassManagerBuilder.h>
 #include <llvm-c/Transforms/Scalar.h>
 #if LLVM_VERSION_MAJOR > 6
 #include <llvm-c/Transforms/Utils.h>
 #endif
-#endif
 
 
-#define LLVMJIT_LLVM_CONTEXT_REUSE_MAX 100
-
 /* Handle of a module emitted via ORC JIT */
 typedef struct LLVMJitHandle
 {
@@ -97,24 +89,23 @@ LLVMTypeRef StructExprState;
 LLVMTypeRef StructAggState;
 LLVMTypeRef StructAggStatePerGroupData;
 LLVMTypeRef StructAggStatePerTransData;
-LLVMTypeRef StructPlanState;
 
 LLVMValueRef AttributeTemplate;
-LLVMValueRef ExecEvalSubroutineTemplate;
+LLVMValueRef FuncStrlen;
+LLVMValueRef FuncVarsizeAny;
+LLVMValueRef FuncSlotGetsomeattrsInt;
+LLVMValueRef FuncSlotGetmissingattrs;
+LLVMValueRef FuncMakeExpandedObjectReadOnlyInternal;
+LLVMValueRef FuncExecEvalSubscriptingRef;
+LLVMValueRef FuncExecEvalSysVar;
+LLVMValueRef FuncExecAggTransReparent;
+LLVMValueRef FuncExecAggInitGroup;
 
-LLVMModuleRef llvm_types_module = NULL;
 
 static bool llvm_session_initialized = false;
 static size_t llvm_generation = 0;
-
-/* number of LLVMJitContexts that currently are in use */
-static size_t llvm_jit_context_in_use_count = 0;
-
-/* how many times has the current LLVMContextRef been used */
-static size_t llvm_llvm_context_reuse_count = 0;
 static const char *llvm_triple = NULL;
 static const char *llvm_layout = NULL;
-static LLVMContextRef llvm_context;
 
 
 static LLVMTargetRef llvm_targetref;
@@ -135,8 +126,6 @@ static void llvm_compile_module(LLVMJitContext *context);
 static void llvm_optimize_module(LLVMJitContext *context, LLVMModuleRef module);
 
 static void llvm_create_types(void);
-static void llvm_set_target(void);
-static void llvm_recreate_llvm_context(void);
 static uint64_t llvm_resolve_symbol(const char *name, void *ctx);
 
 #if LLVM_VERSION_MAJOR > 11
@@ -158,63 +147,6 @@ _PG_jit_provider_init(JitProviderCallbacks *cb)
 	cb->compile_expr = llvm_compile_expr;
 }
 
-
-/*
- * Every now and then create a new LLVMContextRef. Unfortunately, during every
- * round of inlining, types may "leak" (they can still be found/used via the
- * context, but new types will be created the next time in inlining is
- * performed). To prevent that from slowly accumulating problematic amounts of
- * memory, recreate the LLVMContextRef we use. We don't want to do so too
- * often, as that implies some overhead (particularly re-loading the module
- * summaries / modules is fairly expensive). A future TODO would be to make
- * this more finegrained and only drop/recreate the LLVMContextRef when we know
- * there has been inlining. If we can get the size of the context from LLVM
- * then that might be a better way to determine when to drop/recreate rather
- * then the usagecount heuristic currently employed.
- */
-static void
-llvm_recreate_llvm_context(void)
-{
-	if (!llvm_context)
-		elog(ERROR, "Trying to recreate a non-existing context");
-
-	/*
-	 * We can only safely recreate the LLVM context if no other code is being
-	 * JITed, otherwise we'd release the types in use for that.
-	 */
-	if (llvm_jit_context_in_use_count > 0)
-	{
-		llvm_llvm_context_reuse_count++;
-		return;
-	}
-
-	if (llvm_llvm_context_reuse_count <= LLVMJIT_LLVM_CONTEXT_REUSE_MAX)
-	{
-		llvm_llvm_context_reuse_count++;
-		return;
-	}
-
-	/*
-	 * Need to reset the modules that the inlining code caches before
-	 * disposing of the context. LLVM modules exist within a specific LLVM
-	 * context, therefore disposing of the context before resetting the cache
-	 * would lead to dangling pointers to modules.
-	 */
-	llvm_inline_reset_caches();
-
-	LLVMContextDispose(llvm_context);
-	llvm_context = LLVMContextCreate();
-	llvm_llvm_context_reuse_count = 0;
-
-	/*
-	 * Re-build cached type information, so code generation code can rely on
-	 * that information to be present (also prevents the variables to be
-	 * dangling references).
-	 */
-	llvm_create_types();
-}
-
-
 /*
  * Create a context for JITing work.
  *
@@ -231,8 +163,6 @@ llvm_create_context(int jitFlags)
 
 	llvm_session_initialize();
 
-	llvm_recreate_llvm_context();
-
 	ResourceOwnerEnlargeJIT(CurrentResourceOwner);
 
 	context = MemoryContextAllocZero(TopMemoryContext,
@@ -243,8 +173,6 @@ llvm_create_context(int jitFlags)
 	context->base.resowner = CurrentResourceOwner;
 	ResourceOwnerRememberJIT(CurrentResourceOwner, PointerGetDatum(context));
 
-	llvm_jit_context_in_use_count++;
-
 	return context;
 }
 
@@ -254,13 +182,7 @@ llvm_create_context(int jitFlags)
 static void
 llvm_release_context(JitContext *context)
 {
-	LLVMJitContext *llvm_jit_context = (LLVMJitContext *) context;
-
-	/*
-	 * Consider as cleaned up even if we skip doing so below, that way we can
-	 * verify the tracking is correct (see llvm_shutdown()).
-	 */
-	llvm_jit_context_in_use_count--;
+	LLVMJitContext *llvm_context = (LLVMJitContext *) context;
 
 	/*
 	 * When this backend is exiting, don't clean up LLVM. As an error might
@@ -272,18 +194,18 @@ llvm_release_context(JitContext *context)
 
 	llvm_enter_fatal_on_oom();
 
-	if (llvm_jit_context->module)
+	if (llvm_context->module)
 	{
-		LLVMDisposeModule(llvm_jit_context->module);
-		llvm_jit_context->module = NULL;
+		LLVMDisposeModule(llvm_context->module);
+		llvm_context->module = NULL;
 	}
 
-	while (llvm_jit_context->handles != NIL)
+	while (llvm_context->handles != NIL)
 	{
 		LLVMJitHandle *jit_handle;
 
-		jit_handle = (LLVMJitHandle *) linitial(llvm_jit_context->handles);
-		llvm_jit_context->handles = list_delete_first(llvm_jit_context->handles);
+		jit_handle = (LLVMJitHandle *) linitial(llvm_context->handles);
+		llvm_context->handles = list_delete_first(llvm_context->handles);
 
 #if LLVM_VERSION_MAJOR > 11
 		{
@@ -311,10 +233,6 @@ llvm_release_context(JitContext *context)
 
 		pfree(jit_handle);
 	}
-	list_free(llvm_jit_context->handles);
-	llvm_jit_context->handles = NIL;
-
-	llvm_leave_fatal_on_oom();
 }
 
 /*
@@ -332,7 +250,7 @@ llvm_mutable_module(LLVMJitContext *context)
 	{
 		context->compiled = false;
 		context->module_generation = llvm_generation++;
-		context->module = LLVMModuleCreateWithNameInContext("pg", llvm_context);
+		context->module = LLVMModuleCreateWithName("pg");
 		LLVMSetTarget(context->module, llvm_triple);
 		LLVMSetDataLayout(context->module, llvm_layout);
 	}
@@ -462,71 +380,26 @@ llvm_get_function(LLVMJitContext *context, const char *funcname)
 }
 
 /*
- * Return type of a variable in llvmjit_types.c. This is useful to keep types
- * in sync between plain C and JIT related code.
- */
-LLVMTypeRef
-llvm_pg_var_type(const char *varname)
-{
-	LLVMValueRef v_srcvar;
-	LLVMTypeRef typ;
-
-	/* this'll return a *pointer* to the global */
-	v_srcvar = LLVMGetNamedGlobal(llvm_types_module, varname);
-	if (!v_srcvar)
-		elog(ERROR, "variable %s not in llvmjit_types.c", varname);
-
-	typ = LLVMGlobalGetValueType(v_srcvar);
-
-	return typ;
-}
-
-/*
- * Return function type of a variable in llvmjit_types.c. This is useful to
- * keep function types in sync between C and JITed code.
- */
-LLVMTypeRef
-llvm_pg_var_func_type(const char *varname)
-{
-	LLVMValueRef v_srcvar;
-	LLVMTypeRef typ;
-
-	v_srcvar = LLVMGetNamedFunction(llvm_types_module, varname);
-	if (!v_srcvar)
-		elog(ERROR, "function %s not in llvmjit_types.c", varname);
-
-	typ = LLVMGetFunctionType(v_srcvar);
-
-	return typ;
-}
-
-/*
- * Return declaration for a function referenced in llvmjit_types.c, adding it
- * to the module if necessary.
+ * Return declaration for passed function, adding it to the module if
+ * necessary.
  *
- * This is used to make functions discovered via llvm_create_types() known to
- * the module that's currently being worked on.
+ * This is used to make functions imported by llvm_create_types() known to the
+ * module that's currently being worked on.
  */
 LLVMValueRef
-llvm_pg_func(LLVMModuleRef mod, const char *funcname)
+llvm_get_decl(LLVMModuleRef mod, LLVMValueRef v_src)
 {
-	LLVMValueRef v_srcfn;
 	LLVMValueRef v_fn;
 
 	/* don't repeatedly add function */
-	v_fn = LLVMGetNamedFunction(mod, funcname);
+	v_fn = LLVMGetNamedFunction(mod, LLVMGetValueName(v_src));
 	if (v_fn)
 		return v_fn;
 
-	v_srcfn = LLVMGetNamedFunction(llvm_types_module, funcname);
-
-	if (!v_srcfn)
-		elog(ERROR, "function %s not in llvmjit_types.c", funcname);
-
 	v_fn = LLVMAddFunction(mod,
-						   funcname,
-						   LLVMGetFunctionType(v_srcfn));
-	llvm_copy_attributes(v_srcfn, v_fn);
+						   LLVMGetValueName(v_src),
+						   LLVMGetElementType(LLVMTypeOf(v_src)));
+	llvm_copy_attributes(v_src, v_fn);
 
 	return v_fn;
 }
@@ -571,11 +444,8 @@ llvm_copy_attributes(LLVMValueRef v_from, LLVMValueRef v_to)
 	/* copy function attributes */
 	llvm_copy_attributes_at_index(v_from, v_to, LLVMAttributeFunctionIndex);
 
-	if (LLVMGetTypeKind(LLVMGetFunctionReturnType(v_to)) != LLVMVoidTypeKind)
-	{
-		/* and the return value attributes */
-		llvm_copy_attributes_at_index(v_from, v_to, LLVMAttributeReturnIndex);
-	}
+	/* and the return value attributes */
+	llvm_copy_attributes_at_index(v_from, v_to, LLVMAttributeReturnIndex);
 
 	/* and each function parameter's attribute */
 	param_count = LLVMCountParams(v_from);
@@ -624,7 +494,7 @@ llvm_function_reference(LLVMJitContext *context,
 							fcinfo->flinfo->fn_oid);
 		v_fn = LLVMGetNamedGlobal(mod, funcname);
 		if (v_fn != 0)
-			return l_load(builder, TypePGFunction, v_fn, "");
+			return LLVMBuildLoad(builder, v_fn, "");
 
 		v_fn_addr = l_ptr_const(fcinfo->flinfo->fn_addr, TypePGFunction);
 
@@ -634,7 +504,7 @@ llvm_function_reference(LLVMJitContext *context,
 		LLVMSetLinkage(v_fn, LLVMPrivateLinkage);
 		LLVMSetUnnamedAddr(v_fn, true);
 
-		return l_load(builder, TypePGFunction, v_fn, "");
+		return LLVMBuildLoad(builder, v_fn, "");
 	}
 
 	/* check if function already has been added */
@@ -642,7 +512,7 @@ llvm_function_reference(LLVMJitContext *context,
 	if (v_fn != 0)
 		return v_fn;
 
-	v_fn = LLVMAddFunction(mod, funcname, LLVMGetFunctionType(AttributeTemplate));
+	v_fn = LLVMAddFunction(mod, funcname, LLVMGetElementType(TypePGFunction));
 
 	return v_fn;
 }
@@ -653,7 +523,6 @@ llvm_function_reference(LLVMJitContext *context,
 static void
 llvm_optimize_module(LLVMJitContext *context, LLVMModuleRef module)
 {
-#if LLVM_VERSION_MAJOR < 17
 	LLVMPassManagerBuilderRef llvm_pmb;
 	LLVMPassManagerRef llvm_mpm;
 	LLVMPassManagerRef llvm_fpm;
@@ -717,31 +586,6 @@ llvm_optimize_module(LLVMJitContext *context, LLVMModuleRef module)
 	LLVMDisposePassManager(llvm_mpm);
 
 	LLVMPassManagerBuilderDispose(llvm_pmb);
-#else
-	LLVMPassBuilderOptionsRef options;
-	LLVMErrorRef err;
-	const char *passes;
-
-	if (context->base.flags & PGJIT_OPT3)
-		passes = "default<O3>";
-	else
-		passes = "default<O0>,mem2reg";
-
-	options = LLVMCreatePassBuilderOptions();
-
-#ifdef LLVM_PASS_DEBUG
-	LLVMPassBuilderOptionsSetDebugLogging(options, 1);
-#endif
-
-	LLVMPassBuilderOptionsSetInlinerThreshold(options, 512);
-
-	err = LLVMRunPasses(module, passes, NULL, options);
-
-	if (err)
-		elog(ERROR, "failed to JIT module: %s", llvm_error_message(err));
-
-	LLVMDisposePassBuilderOptions(options);
-#endif
 }
 
 /*
@@ -919,24 +763,13 @@ llvm_session_initialize(void)
 	LLVMInitializeNativeAsmPrinter();
 	LLVMInitializeNativeAsmParser();
 
-	if (llvm_context == NULL)
-	{
-		llvm_context = LLVMContextCreate();
-
-		llvm_jit_context_in_use_count = 0;
-		llvm_llvm_context_reuse_count = 0;
-	}
-
 	/*
-	 * When targeting LLVM 15, turn off opaque pointers for the context we
-	 * build our code in.  We don't need to do so for other contexts (e.g.
-	 * llvm_ts_context).  Once the IR is generated, it carries the necessary
-	 * information.
-	 *
-	 * For 16 and above, opaque pointers must be used, and we have special
-	 * code for that.
+	 * When targeting an LLVM version with opaque pointers enabled by
+	 * default, turn them off for the context we build our code in.  We don't
+	 * need to do so for other contexts (e.g. llvm_ts_context).  Once the IR is
+	 * generated, it carries the necessary information.
 	 */
-#if LLVM_VERSION_MAJOR == 15
+#if LLVM_VERSION_MAJOR > 14
 	LLVMContextSetOpaquePointers(LLVMGetGlobalContext(), false);
 #endif
 
@@ -946,14 +779,9 @@ llvm_session_initialize(void)
 	 */
 	llvm_create_types();
 
-	/*
-	 * Extract target information from loaded module.
-	 */
-	llvm_set_target();
-
 	if (LLVMGetTargetFromTriple(llvm_triple, &llvm_targetref, &error) != 0)
 	{
-		elog(FATAL, "failed to query triple %s", error);
+		elog(FATAL, "failed to query triple %s\n", error);
 	}
 
 	/*
@@ -1022,7 +850,7 @@ llvm_session_initialize(void)
 	}
 #endif							/* LLVM_VERSION_MAJOR > 11 */
 
-	on_proc_exit(llvm_shutdown, 0);
+	before_shmem_exit(llvm_shutdown, 0);
 
 	llvm_session_initialized = true;
 
@@ -1046,10 +874,6 @@ llvm_shutdown(int code, Datum arg)
 		return;
 	}
 
-	if (llvm_jit_context_in_use_count != 0)
-		elog(PANIC, "LLVMJitContext in use count not 0 at exit (is %zu)",
-			 llvm_jit_context_in_use_count);
-
 #if LLVM_VERSION_MAJOR > 11
 	{
 		if (llvm_opt3_orc)
@@ -1095,6 +919,26 @@ llvm_shutdown(int code, Datum arg)
 #endif							/* LLVM_VERSION_MAJOR > 11 */
 }
 
+/* helper for llvm_create_types, returning a global var's type */
+static LLVMTypeRef
+load_type(LLVMModuleRef mod, const char *name)
+{
+	LLVMValueRef value;
+	LLVMTypeRef typ;
+
+	/* this'll return a *pointer* to the global */
+	value = LLVMGetNamedGlobal(mod, name);
+	if (!value)
+		elog(ERROR, "type %s is unknown", name);
+
+	/* therefore look at the contained type and return that */
+	typ = LLVMTypeOf(value);
+	Assert(typ != NULL);
+	typ = LLVMGetElementType(typ);
+	Assert(typ != NULL);
+	return typ;
+}
+
 /* helper for llvm_create_types, returning a function's return type */
 static LLVMTypeRef
 load_return_type(LLVMModuleRef mod, const char *name)
@@ -1107,28 +951,19 @@ load_return_type(LLVMModuleRef mod, const char *name)
 	if (!value)
 		elog(ERROR, "function %s is unknown", name);
 
-	typ = LLVMGetFunctionReturnType(value); /* in llvmjit_wrap.cpp */
+	/* get type of function pointer */
+	typ = LLVMTypeOf(value);
+	Assert(typ != NULL);
+	/* dereference pointer */
+	typ = LLVMGetElementType(typ);
+	Assert(typ != NULL);
+	/* and look at return type */
+	typ = LLVMGetReturnType(typ);
+	Assert(typ != NULL);
 
 	return typ;
 }
 
-/*
- * Load triple & layout from clang emitted file so we're guaranteed to be
- * compatible.
- */
-static void
-llvm_set_target(void)
-{
-	if (!llvm_types_module)
-		elog(ERROR, "failed to extract target information, llvmjit_types.c not loaded");
-
-	if (llvm_triple == NULL)
-		llvm_triple = pstrdup(LLVMGetTarget(llvm_types_module));
-
-	if (llvm_layout == NULL)
-		llvm_layout = pstrdup(LLVMGetDataLayoutStr(llvm_types_module));
-}
-
 /*
  * Load required information, types, function signatures from llvmjit_types.c
  * and make them available in global variables.
@@ -1141,6 +976,7 @@ llvm_create_types(void)
 	char		path[MAXPGPATH];
 	LLVMMemoryBufferRef buf;
 	char	   *msg;
+	LLVMModuleRef mod = NULL;
 
 	snprintf(path, MAXPGPATH, "%s/%s", pkglib_path, "llvmjit_types.bc");
 
@@ -1152,36 +988,55 @@ llvm_create_types(void)
 	}
 
 	/* eagerly load contents, going to need it all */
-	if (LLVMParseBitcodeInContext2(llvm_context, buf, &llvm_types_module))
+	if (LLVMParseBitcode2(buf, &mod))
 	{
-		elog(ERROR, "LLVMParseBitcodeInContext2 of %s failed", path);
+		elog(ERROR, "LLVMParseBitcode2 of %s failed", path);
 	}
 	LLVMDisposeMemoryBuffer(buf);
 
-	TypeSizeT = llvm_pg_var_type("TypeSizeT");
-	TypeParamBool = load_return_type(llvm_types_module, "FunctionReturningBool");
-	TypeStorageBool = llvm_pg_var_type("TypeStorageBool");
-	TypePGFunction = llvm_pg_var_type("TypePGFunction");
-	StructNullableDatum = llvm_pg_var_type("StructNullableDatum");
-	StructExprContext = llvm_pg_var_type("StructExprContext");
-	StructExprEvalStep = llvm_pg_var_type("StructExprEvalStep");
-	StructExprState = llvm_pg_var_type("StructExprState");
-	StructFunctionCallInfoData = llvm_pg_var_type("StructFunctionCallInfoData");
-	StructMemoryContextData = llvm_pg_var_type("StructMemoryContextData");
-	StructTupleTableSlot = llvm_pg_var_type("StructTupleTableSlot");
-	StructHeapTupleTableSlot = llvm_pg_var_type("StructHeapTupleTableSlot");
-	StructMinimalTupleTableSlot = llvm_pg_var_type("StructMinimalTupleTableSlot");
-	StructHeapTupleData = llvm_pg_var_type("StructHeapTupleData");
-	StructHeapTupleHeaderData = llvm_pg_var_type("StructHeapTupleHeaderData");
-	StructTupleDescData = llvm_pg_var_type("StructTupleDescData");
-	StructAggState = llvm_pg_var_type("StructAggState");
-	StructAggStatePerGroupData = llvm_pg_var_type("StructAggStatePerGroupData");
-	StructAggStatePerTransData = llvm_pg_var_type("StructAggStatePerTransData");
-	StructPlanState = llvm_pg_var_type("StructPlanState");
-	StructMinimalTupleData = llvm_pg_var_type("StructMinimalTupleData");
+	/*
+	 * Load triple & layout from clang emitted file so we're guaranteed to be
+	 * compatible.
+	 */
+	llvm_triple = pstrdup(LLVMGetTarget(mod));
+	llvm_layout = pstrdup(LLVMGetDataLayoutStr(mod));
 
-	AttributeTemplate = LLVMGetNamedFunction(llvm_types_module, "AttributeTemplate");
-	ExecEvalSubroutineTemplate = LLVMGetNamedFunction(llvm_types_module, "ExecEvalSubroutineTemplate");
+	TypeSizeT = load_type(mod, "TypeSizeT");
+	TypeParamBool = load_return_type(mod, "FunctionReturningBool");
+	TypeStorageBool = load_type(mod, "TypeStorageBool");
+	TypePGFunction = load_type(mod, "TypePGFunction");
+	StructNullableDatum = load_type(mod, "StructNullableDatum");
+	StructExprContext = load_type(mod, "StructExprContext");
+	StructExprEvalStep = load_type(mod, "StructExprEvalStep");
+	StructExprState = load_type(mod, "StructExprState");
+	StructFunctionCallInfoData = load_type(mod, "StructFunctionCallInfoData");
+	StructMemoryContextData = load_type(mod, "StructMemoryContextData");
+	StructTupleTableSlot = load_type(mod, "StructTupleTableSlot");
+	StructHeapTupleTableSlot = load_type(mod, "StructHeapTupleTableSlot");
+	StructMinimalTupleTableSlot = load_type(mod, "StructMinimalTupleTableSlot");
+	StructHeapTupleData = load_type(mod, "StructHeapTupleData");
+	StructTupleDescData = load_type(mod, "StructTupleDescData");
+	StructAggState = load_type(mod, "StructAggState");
+	StructAggStatePerGroupData = load_type(mod, "StructAggStatePerGroupData");
+	StructAggStatePerTransData = load_type(mod, "StructAggStatePerTransData");
+
+	AttributeTemplate = LLVMGetNamedFunction(mod, "AttributeTemplate");
+	FuncStrlen = LLVMGetNamedFunction(mod, "strlen");
+	FuncVarsizeAny = LLVMGetNamedFunction(mod, "varsize_any");
+	FuncSlotGetsomeattrsInt = LLVMGetNamedFunction(mod, "slot_getsomeattrs_int");
+	FuncSlotGetmissingattrs = LLVMGetNamedFunction(mod, "slot_getmissingattrs");
+	FuncMakeExpandedObjectReadOnlyInternal = LLVMGetNamedFunction(mod, "MakeExpandedObjectReadOnlyInternal");
+	FuncExecEvalSubscriptingRef = LLVMGetNamedFunction(mod, "ExecEvalSubscriptingRef");
+	FuncExecEvalSysVar = LLVMGetNamedFunction(mod, "ExecEvalSysVar");
+	FuncExecAggTransReparent = LLVMGetNamedFunction(mod, "ExecAggTransReparent");
+	FuncExecAggInitGroup = LLVMGetNamedFunction(mod, "ExecAggInitGroup");
+
+	/*
+	 * Leave the module alive, otherwise references to function would be
+	 * dangling.
+	 */
+
+	return;
 }
 
 /*
@@ -1321,13 +1176,8 @@ llvm_log_jit_error(void *ctx, LLVMErrorRef error)
 static LLVMOrcObjectLayerRef
 llvm_create_object_layer(void *Ctx, LLVMOrcExecutionSessionRef ES, const char *Triple)
 {
-#ifdef USE_LLVM_BACKPORT_SECTION_MEMORY_MANAGER
 	LLVMOrcObjectLayerRef objlayer =
-		LLVMOrcCreateRTDyldObjectLinkingLayerWithSafeSectionMemoryManager(ES);
-#else
-	LLVMOrcObjectLayerRef objlayer =
-		LLVMOrcCreateRTDyldObjectLinkingLayerWithSectionMemoryManager(ES);
-#endif
+	LLVMOrcCreateRTDyldObjectLinkingLayerWithSectionMemoryManager(ES);
 
 #if defined(HAVE_DECL_LLVMCREATEGDBREGISTRATIONLISTENER) && HAVE_DECL_LLVMCREATEGDBREGISTRATIONLISTENER
 	if (jit_debugging_support)
